#!/bin/bash

. /etc/kona/soundsystem.inc

trace soundsystem-in

if [ ! -z "$(grep x86_64 /etc/kona/os-release)" ]; then
  arch="x86_64"
else
  arch="i386"
fi

mkdir -p $(cat /etc/kona/tmp)

# set timeslice / runtime / clocksource
if [ -f /etc/kona/runtime ]; then
  cat /etc/kona/runtime > /proc/sys/kernel/sched_rt_runtime_us
fi

if [ -f /etc/kona/clocksource ]; then
  cat /etc/kona/clocksource > /sys/devices/system/clocksource/clocksource0/current_clocksource
fi
  
if [ -f /etc/kona/timeslice ]; then
  timeslice="$(cat /etc/kona/timeslice)"
  echo "$timeslice" > /proc/sys/kernel/sched_rr_timeslice_ms
fi

if [ -f /etc/kona/maxuserfreq ]; then
  cat /etc/kona/maxuserfreq > /proc/sys/dev/hpet/max-user-freq
fi

# set pci freq
#/etc/kona/setpci.sh > /dev/null

# set i/o scheduler
if [ -f /etc/kona/iosched ]; then
  iosched=$(cat /etc/kona/iosched)
  for i in $(ls /sys/block)
  do
    if [ "$(echo "$i"|cut -b1-2)" = "sd" ]; then
      if ! grep -q "$iosched" /sys/block/"$i"/queue/scheduler ; then
        if [ "$iosched" = "noop" ]; then
          iosched="none"
        else
          if [ "$iosched" = "cfq" ]; then
            iosched="bfq"
          else
            if [ "$iosched" = "deadline" ]; then
              iosched="mq-deadline"
            fi
          fi
        fi
      fi  
      if grep -q "$iosched" /sys/block/"$i"/queue/scheduler ; then
        echo "$iosched" > /sys/block/"$i"/queue/scheduler
      fi
    fi
  done
fi

# set io cache
[ -f /etc/kona/dirty_ratio ] && cat /etc/kona/dirty_ratio > /proc/sys/vm/dirty_ratio
[ -f /etc/kona/dirty_background_ratio ] && cat /etc/kona/dirty_background_ratio > /proc/sys/vm/dirty_background_ratio

if [ -f /etc/kona/rmem ]; then
# set TCP read buffer size
  minsz=$(cat /proc/sys/net/ipv4/tcp_rmem | awk '{print $1}')
  maxsz=$(cat /proc/sys/net/ipv4/tcp_rmem | awk '{print $3}')
  rmem="$(cat /etc/kona/rmem)"
  if [ $rmem -lt $minsz ]; then
    rmem=$minsz
  fi
  echo $minsz $rmem $maxsz > /proc/sys/net/ipv4/tcp_rmem
  trace rmem=$rmem
fi

# set mtu
if [ -f /etc/kona/mtu ]; then
  ifconfig|grep mtu|grep -v ^lo|cut -d':' -f1 | while read line; do
    ifconfig $line mtu $(cat /etc/kona/mtu)
  done
fi

# disable/enable alsa of audacious
#if [ "$(cat /etc/kona/soundsystem)" = "pulse" ]; then
#  if [ -f /usr/lib/$arch-linux-gnu/audacious/Output/alsa.so ]; then
#    mv /usr/lib/$arch-linux-gnu/audacious/Output/alsa.so /usr/local/etc/kona
#  fi
#else
  if [ ! -f /usr/lib/$arch-linux-gnu/audacious/Output/alsa.so ]; then
    mv /usr/local/etc/kona/alsa.so /usr/lib/$arch-linux-gnu/audacious/Output
  fi
#fi

# init ram
if [ -f /etc/kona/initram ]; then
  /etc/kona/initram.sh $(cat /etc/kona/initram)
fi

trace soundsystem-tuneup-alsa
# tune up alsa lib
if [ -f /etc/kona/hqalsa ] && [ -d /etc/kona/alsalib ]; then
  if [ ! -f /etc/kona/alsalib/libasound.so.2.0.0 ]; then
    cp /usr/lib/$arch-linux-gnu/libasound.so.2.0.0 /etc/kona/alsalib
    if [ ! -z "$(grep avx2 /proc/cpuinfo)" ]; then
      cp /etc/kona/alsalib/libasound.so.2.0.0-avx /usr/lib/$arch-linux-gnu/libasound.so.2.0.0
    else
      cp /etc/kona/alsalib/libasound.so.2.0.0-sse2 /usr/lib/$arch-linux-gnu/libasound.so.2.0.0
    fi
  fi
else
  if [ -f /etc/kona/alsalib/libasound.so.2.0.0 ]; then
    mv /etc/kona/alsalib/libasound.so.2.0.0 /usr/lib/$arch-linux-gnu/libasound.so.2.0.0
 fi
fi 

# set systemd lib to ram when deep or soft mode
if [ ! -z "$(grep deep /etc/kona/accelator)" ] || [ "$(cat /etc/kona/depth)" = "deep" ] || [ "$(cat /etc/kona/soundmode)" = "deep" ] || [ "$(cat /etc/kona/soundmode)" = "soft" ]; then
  if [ -f $tmp/libsystemd.so.0 ]; then
    refreshfile2 $tmp/libsystemd.so.0 libsystemd.so.0 min
  else
    cp2ram /usr/lib/$arch-linux-gnu/libsystemd.so.0 /usr/lib/$arch-linux-gnu
  fi
fi

# hifi mpd
if [ -f /etc/kona/hqmpd ] && [ "$(cat /etc/kona/hqmpd)" = "max" ]; then
  cp2ram /usr/lib/x86_64-linux-gnu/libmpdclient.so.2 /usr/lib/$arch-linux-gnu
fi

# boost base
if [ -f /etc/kona/baseboost ]; then
  cp2ram /usr/lib/$arch-linux-gnu/libupower-glib.so.3 /usr/lib/$arch-linux-gnu
fi
trace soundsystem-ldconfig
rm -f /etc/ld.so.cache
ldconfig

trace soundsystem-server-to-ram
# set server to ram
/etc/kona/setramserver.sh

# set player to ram
/etc/kona/setplayertoram.sh

# restart bluetooth
systemctl restart bluetooth
trace "Restart bluetooth"

# restart upmpdcli
if [ -f /etc/kona/hqmpd ]; then
  if [ ! -z "$(pgrep upmpdcli)" ]; then
    systemctl restart upmpdcli
    trace "Restart upmpdcli"
  fi
fi

# check fastsoundriver
if [ -f /etc/kona/fastsoundriver ]; then
  if [ -z "$(modinfo snd-pcm|grep signature:)" ]; then
    touch /etc/kona/fastdriverdoing
  else
    rm -f /etc/kona/fastdriverdoing
  fi
fi
if [ -f /etc/kona/hqusbaudiodriver ]; then
  if [ -z "$(modinfo snd-usb-audio|grep signature:)" ]; then
    touch /etc/kona/hqusbaudiodoing
  else
    rm -f /etc/kona/hqusbaudiodoing
  fi
fi

if [ -f /etc/kona/headless ]; then
  systemctl start xrdp
fi

trace soundsystem-out
touch /etc/kona/soundset
